@@ -83,6 +83,10 @@ subset.SpatialPoints <- function(x, subset, select, drop = FALSE, ...) {
 	res
 }
 
+row.names.SpatialPoints <- function(x) {
+    dimnames(slot(x, "coords"))[[1]]
+}
+
 #"[.SpatialPoints" =  function(x, i, j, ..., drop = T) {
 setMethod("[", "SpatialPoints", function(x, i, j, ..., drop = TRUE) {
 	if (!missing(j))
